/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 Justin Kirby
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 */
#include "oapr/oapr.hpp"
#include "oapr/oapr_buildconfig.hpp"
#include <iostream>
#include <iomanip>
#include <memory>
#include <string>
#include <assert.h>

FILE_VERSION("$Revision: 1.3.2.1 $",oapr_buildconfig_cpp);

namespace oapr
{
  version_stamp::version_stamp(const char* file,
			       const char* date,
			       const char* time,
			       const char* ver)
    : m_fileName(file)
      , m_buildDate(date)
      , m_buildTime(time)
      , m_version(ver)
  {
    build_config::Instance().AddFile(this);
  }


  const char* version_stamp::file_name() const
  {
    return m_fileName;
  }
  

  std::string version_stamp::version() const
  {
    std::string temp(m_version);
    std::string::size_type first = temp.find_first_of("0123456789");
    std::string::size_type last = temp.find_last_of("0123456789");
    if(first==std::string::npos)return "";
    if(last==std::string::npos)return temp;
    return temp.substr(first, last-first + 1);
  }


  const char* version_stamp::build_date() const
  {
    return m_buildDate;
  }
  
  
  const char* version_stamp::build_time() const
  {
    return m_buildTime;
  }


  bool version_stamp::operator<(const version_stamp& rhs) const
  {
    return strcmp(m_fileName, rhs.m_fileName) < 0;
  }


  bool VersionStampPtrLess::operator ()(const version_stamp* lhs,
					const version_stamp* rhs)
  {
    return (*lhs) < (*rhs);
  }
  

  //*****************************************************************************|
  //*   build_config Implementation
//*****************************************************************************|
  
  build_config& build_config::Instance()
  {
    static std::auto_ptr<build_config>
      instance(new build_config);

    return *instance;
  }
  

  build_config::build_config()
  {
  }


  void build_config::Output(std::ostream& out) const
  {
    using namespace std;
    
    typedef VersionInfo::const_iterator It;
    ios::fmtflags oldFlags = out.flags();
  
    out << setw(41) << left << "File Name"
	<< setw(12) << right << "Version"
	<< setw(15) << right << "Build Date" 
	<< setw(10) << right << "Time" << '\n';

    out << "-------------------------------------------"
      "------------------------------------\n";
    
    for (It it(mConfig.begin()), end(mConfig.end()); it != end; ++it) {
      const version_stamp& vs = **it;
      
      out << setw(41) << left << vs.file_name()
	  << setw(12) << right << vs.version() 
	  << setw(15) << right << vs.build_date()
	  << setw(10) << right << vs.build_time()
	  << '\n';
    }

    out.flags(oldFlags);
  }


  void build_config::AddFile(const version_stamp* fileStamp)
  {
    assert(fileStamp);
    mConfig.insert(fileStamp);
  }
  

  build_config::const_iterator build_config::begin() const
  {
    return mConfig.begin();
  }
  

  build_config::const_iterator build_config::end() const
  {
    return mConfig.end();
  }
};
