/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 Justin Kirby
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 */
#ifndef OAPR_BUILDCONFIG_HPP
#define OAPR_BUILDCONFIG_HPP
#include <iosfwd>
#include <string>
#include <set>
#include "oapr/oapr.hpp"

namespace oapr
{
  class LIBOAPR_API version_stamp
  {
  public: 
    version_stamp(const char* file,
                 const char* date,
                 const char* time,
                 const char* ver);

    const char* file_name() const;
    std::string version() const;
    const char* build_date() const;
    const char* build_time() const;

    bool operator<(const version_stamp& rhs) const;

  private:
    const char* m_fileName;
    const char* m_buildDate;
    const char* m_buildTime;
    const char* m_version;
  };

  struct VersionStampPtrLess
  {
    bool operator()(const version_stamp* lhs, const version_stamp* rhs);
  };

  class build_config
  {
  private:
    typedef std::set<const version_stamp*, VersionStampPtrLess> VersionInfo;
  
  public:
    typedef VersionInfo::const_iterator const_iterator;  
    
  public:
    static build_config& Instance();

    void Output(std::ostream& out) const;
    void AddFile(const version_stamp* fileStamp);

    const_iterator begin() const;
    const_iterator end() const;
    
  private:
    // Private for Singleton
    build_config();
    
    // Unimplemented
    build_config(const build_config&);
    build_config& operator=(const build_config&);
    
  private:
   VersionInfo mConfig;
  };
  
  
};
#define FILEVERHELP(VER, NAME) \
  static oapr::version_stamp vs##NAME(__FILE__, __DATE__, __TIME__, VER)
#define FILE_VERSION(VER, NAME) FILEVERHELP(VER, NAME)

FILE_VERSION("$Revision: 1.2 $", BuildConfig_h);

#endif
