/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 Justin Kirby
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 */
#ifndef OAPR_EXC_RUNTIME_HPP
#define OAPR_EXC_RUNTIME_HPP

#include "oapr/oapr_buildconfig.hpp"
#include "oapr_exc_base.hpp"

FILE_VERSION("$Revision: 1.3 $",oapr_exc_runtime_hpp);

namespace oapr
{

  class runtime_error : public exception_base
  {
  public:
    runtime_error(int line,
	    std::string method,
	    std::string file,
	    std::string msg,
	    int level = 0)
      : exception_base(line,method,file,msg,level){}
    runtime_error(const runtime_error &r,
	    int line,
	    std::string method,
	    std::string file,
	    std::string src,
	    std::string msg,
	    int level = 0)
      : exception_base(r,line,method,file,src,msg,level){}
    virtual ~runtime_error() throw(){}
  };

  class range_error : public runtime_error
  {
  public:
    range_error(int line,
	       std::string method,
	       std::string file,
	       std::string msg,
	       int level = 0)
      : runtime_error(line,method,file,msg,level){}
    range_error(const range_error&re,
		int line,
	       std::string method,
	       std::string file,
	       std::string src,
	       std::string msg,
	       int level = 0)
      : runtime_error(re,line,method,file,src,msg,level){}
    virtual ~range_error() throw(){}
  };
  /**\todo evaluate need for overflow/underflow exceptions*/
  class not_implemented : public runtime_error
  {
  public:
    not_implemented(int line,
	       std::string method,
	       std::string file,
	       std::string msg,
	       int level = 0)
      : runtime_error(line,method,file,msg,level){}
    not_implemented(const not_implemented& ni,
		int line,
	       std::string method,
	       std::string file,
	       std::string src,
	       std::string msg,
	       int level = 0)
      : runtime_error(ni,line,method,file,src,msg,level){}
    virtual ~not_implemented() throw(){}
  };
  //**\todo consider renaming. potentially useful local variable name.*/
  class init_error : public runtime_error
  {
  public:
    init_error(int line,
	       std::string method,
	       std::string file,
	       std::string msg,
	       int level = 0)
      : runtime_error(line,method,file,msg,level){}
    init_error(const not_implemented& ni,
	       int line,
	       std::string method,
	       std::string file,
	       std::string src,
	       std::string msg,
	       int level = 0)
      : runtime_error(ni,line,method,file,src,msg,level){}
    virtual ~init_error() throw(){}
  };
  class invalid_state : public runtime_error
  {
  public:
    invalid_state(int line,
	       std::string method,
	       std::string file,
	       std::string msg,
	       int level = 0)
      : runtime_error(line,method,file,msg,level){}
    invalid_state(const not_implemented& ni,
	       int line,
	       std::string method,
	       std::string file,
	       std::string src,
	       std::string msg,
	       int level = 0)
      : runtime_error(ni,line,method,file,src,msg,level){}
    virtual ~invalid_state() throw(){}
  };
};
#endif
