/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 Justin Kirby
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 */
#include "oapr/oapr.hpp"
#include "oapr/oapr_buildconfig.hpp"
#include "boost/scoped_ptr.hpp"
#include "oapr/oapr_io_socketinfo.hpp"

FILE_VERSION("$Revision: 1.3.2.1 $",oapr_io_socketinfo_cpp);

namespace oapr
{

    void sockinfo::create(const std::string& hostname,
	     apr_int32_t family,
	     apr_port_t port,
	     apr_int32_t flags)
    {
      apr_sockaddr_info_get(&m_addrt,
			    hostname.c_str(),
			    family,port,flags,m_pool->getp());
    }
    std::string sockinfo::hostname()const
    {
      std::string rv;
      if(m_addrt){
	char *tmpnm=0;
	apr_getnameinfo(&tmpnm,m_addrt,0);
	rv=tmpnm;
	delete [] tmpnm;
      }else{
	char tmpnm[APRMAXHOSTLEN+1];
	apr_gethostname(tmpnm,APRMAXHOSTLEN,m_pool->getp());	
	rv = tmpnm;
      }	
      return rv;
    }
    std::string sockinfo::service()const
    {
      std::string rv;
      if(m_addrt && m_addrt->servname){
	rv=m_addrt->servname;
      }
      return rv;
    }
    apr_port_t sockinfo::port()const
    {
      apr_port_t rv;
      if(!m_addrt){
	/**\todo throw here if the addr is not set*/
	return 0;
      }
      apr_sockaddr_port_get(&rv,m_addrt);
      return rv;
    }
  apr_status_t sockinfo::port(apr_port_t p)
  {
    if(!m_addrt){
      /**\todo throw here if the addr is not set*/
      return 0;
    }
    return apr_sockaddr_port_set(m_addrt,p);
  }

    int sockinfo::family()const
    {
      if(m_addrt){
	return m_addrt->family;
      }else{
	return -1;
      }
    }

    std::string sockinfo::ip()const
    {
      char *tmpip=0;
      std::string rv;
      apr_sockaddr_ip_get(&tmpip,m_addrt);
      rv=tmpip;
      delete [] tmpip;
      return rv;
    }
}
