/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2003 Matt Carter. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 */
#include <iostream>
#include "oapr/oapr_app.hpp"
#include "oapr/oapr_pool.hpp"
#include "oapr/oapr_getopt.hpp"
#include "oapr/oapr_exception.hpp"
#include "oapr/oapr_exc_utils.hpp"

using namespace std;
using namespace oapr;

static const static_opts args[6] = {
  // name, optch, has_args, description
  {"force", 'f', 0, "Force stuff"},
  {"verbose", 'v', 0, "Verbose"},
  {"log", 'l', 1, "Log to file"},
  {"with-fries", 'w', 0, "Include large fries"},
  {"number", 'n', 1, "Mandatory integer parameter"},
  END_OPTS
};

int main(int argc, char *argv[])
{
  const char * const *newargv = argv;
  oapr_app app("getopt test", &argc, &newargv);
  pool::pool_ptr apl(new pool);
  try{

    /* Test args */
    int largc = 5;
    const char * const largv[] = {"testprog", "-fv", "-l", "logfile.txt", "-n2"};
    bool force = false, verbose = false, fries = false;
    int number;
    string logfile;

    cmdline_processor a(apl,largc, largv, args);
    while(a.getopt())
    {
      switch(a.opt())
      {
        case 'f':
          force = true; break;
        case 'v':
          verbose = true; break;
        case 'l':
          logfile = a.arg(); break;
        case 'w':
          cout << "You get fries with that." << endl;
          fries = true;
          break;
        case 'n':
          number = a.arg_int();
          break;
        default:
          cerr << "Unknown option parsed."<< endl;
          return 1;
      }
    }

    if(!verbose || !force || logfile != "logfile.txt" || fries || number != 2) return 1;
    
    usage(args, app.name());
    
  }OAPR_PRINT(&std::cerr,return 1);
  return 0;
}
