/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 Justin Kirby
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 */
#ifndef OAPR_LOGMESSAGE_HPP
#define OAPR_LOGMESSAGE_HPP
#include "oapu/oapu.hpp"
namespace oapr
{
  enum Log_Level{
    Log_Emerg=0,
    Log_Alert,
    Log_Crit,
    Log_Error,
    Log_Warning,
    Log_Notice,
    Log_Info,
    Log_Debug
  };
  class LIBOAPU_API log_message
  {
  public:
    log_message(int line,
		std::string method,
		std::string file,
		std::string msg,
		Log_Level level=Log_Debug)
    : m_line(line),
      m_method(method),
      m_file(file),
      m_message(msg),
      m_level(level)
    {
    }
    virtual ~log_message()
    {
    }
    virtual const int line() const throw() {
      return m_line;
    }
    virtual const char* what() const throw() {
      return m_message.c_str();
    }
    virtual const char* method() const throw() {
      return m_method.c_str();
    }
    virtual const char* file() const throw() {
      return m_file.c_str();
    }
    virtual const Log_Level level() const throw() {
      return m_level;
    }
    template<typename char_t,typename traits_t>
    friend std::basic_ostream<char_t,traits_t>&
    operator<<(std::basic_ostream<char_t,traits_t> &os,
	       const log_message& lm)
    {
      if(lm.level()<=loglvl){
	os<<lm.file()<<":"<<lm.line()<<" "
	  <<lm.what();
      }
      return os;
    }
  private:
    log_message(const log_message& lm){}
  private:
    int m_line;
    std::string m_method;
    std::string m_file;
    std::string m_message;
    Log_Level m_level;
  };
};
#endif
